/*
 * Copyright (c) 2009-2012 Xilinx, Inc.  All rights reserved.
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/*
 * MEAS_MS5637_Main.c: Console Application for Testing the MS5637
 *
 * This application configures UART 16550 to baud rate 9600.
 * PS7 UART (Zynq) is not initialized by this application, since
 * bootrom/bsp configures it to baud rate 115200
 *
 * ------------------------------------------------
 * | UART TYPE   BAUD RATE                        |
 * ------------------------------------------------
 *   uartns550   9600
 *   uartlite    Configurable only in HW design
 *   ps7_uart    115200 (configured by bootrom/bsp)
 */

#include <stdio.h>
#include <unistd.h>
#include "platform.h"
#include "xparameters.h"
//#include "sleep.h"
#include "ms5637.h"

void ms5637_main_menu(void);

int main()
{

    char key_input;
    char prom_read_flag=0;
    int i;
    ms5637_status stat;
    float temperature;
    float pressure;

    //Initialize the UART
    init_platform();

    // Set the AXI address of the IIC core
    ms5637_init(XPAR_AXI_IIC_JC_BASEADDR);

    // Display the main menu
    ms5637_main_menu();

    // Infinite loop
    while(1){

        // Get keyboard input
        read(1, (char*)&key_input, 1);

        if(key_input == '1'){         //If the '1' key is pressed

            // Send the reset command to the MS5637
            printf("\n");
            printf("Resetting MS5637...\n");
            stat = ms5637_reset();

            // Display the status returned from the reset operation
            printf("MS5637 Reset Complete with status: ");
            if(stat==ms5637_status_ok)
                printf("Ok.\n");
            if(stat==ms5637_status_i2c_transfer_error)
                printf("Transfer Error.\n");

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            ms5637_main_menu();

        }else if(key_input == '2'){         // If the '2' key is pressed

            // Read the PROM coefficients from the MS5637
            printf("\n");
            printf("Reading PROM Coefficients...\n");
            stat = ms5637_read_prom();

            // Display status returned from read_prom operation
            // and display prom values if successful
            printf("Read PROM Complete with status: ");
            if(stat==ms5637_status_ok){
                prom_read_flag=1;
                printf("Ok.\n");
                printf("\n");
                printf("_________________________________________________\n");
                printf("|   PROM Addr   |Coeff (Base 10)|  Coeff (Hex)  |\n");
                printf("|---------------+---------------+---------------|\n");
                for(i=0;i<7;i++){
                    printf("|\t%d\t|     %5d\t|     0x%4X\t|\n",i,(unsigned int)ms5637_prom_coeffs[i],(unsigned int)ms5637_prom_coeffs[i]);
                }
            }else if(stat==ms5637_status_i2c_transfer_error){
                printf("Transfer Error.\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            ms5637_main_menu();

        }else if(key_input == '3'){

        	// Display resolution selection menu
            printf("\n");
            printf("Select a resolution (over-sampling rate):\n");
            printf("  (1)   - 256\n");
            printf("  (2)   - 512\n");
            printf("  (3)   - 1024\n");
            printf("  (4)   - 2048\n");
            printf("  (5)   - 4096\n");
            printf("  (6)   - 8192\n");

            // Get keyboard input ignoring keypresses that are not '1' or '2' or '3' or '4' or '5' or '6'
            read(1, (char*)&key_input, 1);
            while(key_input!='1' && key_input!='2' && key_input!='3' && key_input!='4' && key_input!='5' && key_input!='6'){
                read(1, (char*)&key_input, 1);
            }

            if(key_input == '1'){           // If the '1' key is pressed
                // Set OSR to 256
                ms5637_resolution = ms5637_resolution_osr_256;
                printf("\nSet MS5637 Over-Sampling Rate to 256\n");
            }else if(key_input == '2'){     // If the '2' key is pressed
                // Set OSR to 512
                ms5637_resolution = ms5637_resolution_osr_512;
                printf("\nSet MS5637 Over-Sampling Rate to 512\n");
            }else if(key_input == '3'){     // If the '3' key is pressed
                // Set OSR to 1024
                ms5637_resolution = ms5637_resolution_osr_1024;
                printf("\nSet MS5637 Over-Sampling Rate to 1024\n");
            }else if(key_input == '4'){     // If the '4' key is pressed
                // Set OSR to 2048
                ms5637_resolution = ms5637_resolution_osr_2048;
                printf("\nSet MS5637 Over-Sampling Rate to 2048\n");
            }else if(key_input == '5'){     // If the '5' key is pressed
                // Set OSR to 4096
                ms5637_resolution = ms5637_resolution_osr_4096;
                printf("\nSet MS5637 Over-Sampling Rate to 4096\n");
            }else if(key_input == '6'){     // If the '6' key is pressed
                // Set OSR to 8192
                ms5637_resolution = ms5637_resolution_osr_8192;
                printf("\nSet MS5637 Over-Sampling Rate to 8192\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            ms5637_main_menu();

        }else if(key_input == '4'){         // If the '4' key is pressed

            if(prom_read_flag==0){          // PROM was not yet read--cannot read temperature and pressure yet
                printf("PROM Coefficients have not yet been read. Cannot complete temperature/pressure read.\n");
            }else{                          // PROM has been read--continue on to read temperature and pressure

				// Read one temperature value and one pressure value
				printf("\n");
				printf("Reading a Temperature Value and a Pressure Value...\n");
				stat = ms5637_read_temperature_and_pressure(&temperature, &pressure);

				// Display the status returned from the read_temperature_and_pressure
				// operation and display the temperature and pressure if successful
				printf("Temperature and Pressure Read Complete with status: ");
				if(stat==ms5637_status_ok){
					printf("Ok.\n");
					printf("Temperature : %5.2f%cC, \tPressure : %6.2fhPa",temperature,248,pressure);
				}else if(stat==ms5637_status_i2c_transfer_error){
					printf("Transfer Error.");
				}
				printf("\n");

            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            ms5637_main_menu();

        }else if(key_input == '5'){         // If the '5' key is pressed

        	if(prom_read_flag==0){          // PROM was not yet read--cannot read temperature and pressure yet
                printf("PROM Coefficients have not yet been read. Cannot complete temperature/pressure read.\n");
            }else{                          // PROM has been read--continue on to read temperature and pressure

				// Read 20 temperature values at ~2 per second
				printf("\n");
				printf("Reading 20 Temperature and Pressure Value Pairs...\n");
				for(i=0;i<20;i++){
					stat = ms5637_read_temperature_and_pressure(&temperature, &pressure);
					if(stat==ms5637_status_ok){
						printf("%2d: Temperature : %5.2f%cC, \tPressure : %6.2fhPa",i+1,temperature,248,pressure);
					}else if(stat==ms5637_status_i2c_transfer_error){
						printf("%2d: Transfer Error.", i+1);
					}
					printf("\n");
					usleep( (500-MS5637_CONV_DELAY_OSR_8192)*1000 );
				}

            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            ms5637_main_menu();

        }else if(key_input == 27){    // If the 'ESC' key is pressed

            // Print done and exit.
            printf("Done.\n");
            break;

        }else{                      // If some other key is pressed

            // Redisplay the main menu
            ms5637_main_menu();

        }

    }

    return 0;

}

void ms5637_main_menu(void){

    //Clear the screen
    printf("\033[2J");

    //Display the main menu
    printf("*******************************************\n");
    printf("****      Measurement Specialties      ****\n");
    printf("*******************************************\n");

    printf("\n");
    printf("    MS5637 - Barometric Pressure Sensor    \n");
    printf("-------------------------------------------\n");

    printf("\n");
    printf("Select a task:\n");
    printf("  (1)   - Reset\n");
    printf("  (2)   - Read PROM Coefficients\n");
    printf("  (3)   - Set Over-Sampling Rate\n");
    printf("  (4)   - Read Temperature and Pressure Once\n");
    printf("  (5)   - Read Temperature and Pressure 20 Times\n");
    printf(" (ESC)  - Quit\n");
    printf("\n");

    return;
}
